package de.intevation.flys.artifacts.services.meta;

import java.util.Map;
import java.util.List;
import java.util.HashMap;
import java.util.ArrayList;

import javax.xml.xpath.XPathVariableResolver;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;

public class StackFrames
implements   XPathVariableResolver
{
    private static Logger log = Logger.getLogger(StackFrames.class);

    protected List<Map<String, Object>> frames;

    public StackFrames() {
        frames = new ArrayList<Map<String, Object>>();
    }

    public StackFrames(Map<String, Object> initialFrame) {
        this();
        if (initialFrame != null) {
            frames.add(new HashMap<String, Object>(initialFrame));
        }
    }

    public void enter() {
        frames.add(new HashMap<String, Object>());
    }

    public void leave() {
        frames.remove(frames.size()-1);
    }

    public void put(String key, Object value) {
        int N = frames.size();
        if (N > 0) {
            frames.get(N-1).put(key, value);
        }
    }

    public void put(String [] keys, Object [] values) {
        Map<String, Object> top = frames.get(frames.size()-1);
        for (int i = 0; i < keys.length; ++i) {
            top.put(keys[i], values[i]);
        }
    }

    public boolean containsKey(String key) {
        for (int i = frames.size()-1; i >= 0; --i) {
            if (frames.get(i).containsKey(key)) {
                return true;
            }
        }
        return false;
    }

    public Object get(String key) {
        return get(key, null);
    }

    public Object get(String key, Object def) {

        for (int i = frames.size()-1; i >= 0; --i) {
            Map<String, Object> frame = frames.get(i);
            if (frame.containsKey(key)) {
                return frame.get(key);
            }
        }

        return def;
    }

    @Override
    public Object resolveVariable(QName variableName) {
        if (log.isDebugEnabled()) {
            log.debug("resolve var: " + variableName);
        }
        return get(variableName.getLocalPart());
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
